local L = Capitalist.Locals

local function PrettyCurrency(copper)
	local text = "";
	
	local gold = floor( copper / COPPER_PER_GOLD );
	if gold > 0 then
		text = ("%s%d|cffffd700g|r"):format(text, gold);
	end
	
	local silver = floor( ( copper % COPPER_PER_GOLD ) / COPPER_PER_SILVER );
	if silver > 0 then
		text = ("%s%d|cffc7c7cfs|r"):format(text, silver);
	end
	
	local copper = floor( copper % COPPER_PER_SILVER );
	if copper > 0 or text == "" then
		text = ("%s%d|cffeda55fc|r"):format(text, copper);
	end
	
	return text;
end

-- StartAuction overwriting

local originalStartAuction = StartAuction;
function StartAuction(bid, buyout, time, stackSize, numStacks)
	local itemLink = select(2, GetItemInfo(GetAuctionSellItemInfo())); -- GetAuctionSellItemInfo first result is an itemName, but GetItemInfo allows itemNames as long as the item is in your current inventory
	StaticPopupDialogs["StartAuctionPostConfirm"] = {
		text = ("Post %d auctions of %dx%s at %s bid and %s buyout?"):format(numStacks or 0, stackSize or 0, itemLink or "", PrettyCurrency(bid or 0), PrettyCurrency(buyout or 0)),
		button1 = YES,
		button2 = NO,
		OnAccept = function(self)
			self.accepted = true;
			
--			self:Hide();
		end,
		OnCancel = function(self)
			self.accepted = false;
		end,
		OnHide = function(self)
			if self.accepted then
				originalStartAuction(bid, buyout, time, stackSize, numStacks);
			else
				--Capitalist:PostCallback(self.accepted)
				Capitalist:PostCallback(false)
			end
		end,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
	};
	StaticPopup_Show("StartAuctionPostConfirm");
end

-- CancelAuction overwriting

local originalCancelAuction = CancelAuction;

-- Queue auctions up (QA sends them in a large burst, we can't click yes that fast)
local auctionsToCancel = {};

local function DoCancelAuction()
	if auctionsToCancel[table.getn(auctionsToCancel)] then
		StaticPopup_Show("CancelAuctionConfirm");
	end
end

-- This actually overwrites the canceling function
function CancelAuction(num)
	-- Queue up
	table.insert(auctionsToCancel, num);
	
	-- Show the dialog
	DoCancelAuction();
end

-- Actually do the cancel
local function CancelNow()
	-- Sort from high to low
	-- We do this because auction ids are based on item position in your auction list, e.g. top item is 1, second is 2, etc.
	-- if we cancel 1, 2 would turn into 1. However, if we cancel 2, then one still stays 1.
	-- Capiche?
	table.sort(auctionsToCancel, function(a, b)
		return a < b;
	end);
	
	originalCancelAuction(auctionsToCancel[table.getn(auctionsToCancel)]);
	
	table.remove(auctionsToCancel, table.getn(auctionsToCancel));
end

StaticPopupDialogs["CancelAuctionConfirm"] = {
	text = "Cancel of auctions is being queued, please wait a split second...\n\nDo you wish to proceed canceling these auctions?",
	button1 = YES,
	button2 = NO,
	OnAccept = CancelNow,
	OnHide = DoCancelAuction, -- Show the dialog for the next auction, this can't be done in the OnAccept as it will show this dialog while the very end of OnAccept will hide it again, thus eventually staying hidden
	OnCancel = function(self, reason)
		if reason == "clicked" then
			table.wipe(auctionsToCancel);
		end
		self:Hide();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
};
