local L = Capitalist.Locals

function Capitalist:ApplyFilters()
	self:DebugEntry("ApplyFilters")

	-- reset all filters first
	for recipe in pairs(self.db.profile.recipes) do
		self.db.profile.recipes[recipe].isUsed = nil
	end
	
	-- deactivate reagents based on global filters
	for reagent, data in pairs(self.db.profile.reagents) do
		local isUsed
	
		if (data.itemId >= 34052 and data.itemId <= 34057) then
			isUsed = self.db.profile.filters.global.Northrend
		elseif (data.itemId >= 22445 and data.itemId <= 22450) then
			isUsed = self.db.profile.filters.global.BurningCrusade
		else
			local lastWord = reagent:match("[%a ]+ (%a+)")
			if (lastWord == L["Dust"] or lastWord == L["Essence"] or lastWord == L["Shard"] or lastWord == L["Crystal"]) then
				isUsed = self.db.profile.filters.global.Classic
			else
				isUsed = nil -- don't know for sure where that item is from
			end
		end	

		data.isUsed = isUsed
		if (isUsed ~= nil) then
			data.isFiltered = not isUsed
		end
	end
	
	-- deactivate recipes based on deactivated reagents or filters
	for recipe, data in pairs(self.db.profile.recipes) do
		local isUsed = nil
	
		for reagent in pairs(data.reagents) do
			local reagentData = self.db.profile.reagents[reagent]
			if (reagentData.isUsed == true and reagentData.isFiltered == false) then
				if (isUsed == nil) then
					isUsed = true
				end
			end
		end
		
		if (isUsed ~= true and self.db.profile.filters.global.Classic == false) then
			data.isUsed = false
		else
			data.isUsed = true
		end
	end
	
	-- activate or deactivate previously unknown reagents based on recipes
	for reagent, data in pairs(self.db.profile.reagents) do
		if (data.isUsed == nil) then
			local isUsed = false
			
			for recipe, recipeData in pairs(self.db.profile.recipes) do
				if (isUsed == false and (recipeData.isUsed == true and recipeData.isFiltered == false)) then
					for recipeReagent in pairs(recipeData.reagents) do
						if (recipeReagent == reagent) then
							isUsed = true
						end
					end
				end
			end
			
			data.isUsed = isUsed
		end
	end
	
	self:BuildList()
	self:UpdateFilterOptions()

	self:DebugExit("ApplyFilters")
end

-- scans the Enchanting trade skill frame and adds all recipes the player knows to the recipes-table
function Capitalist:ScanTradeSkillFrame()
	self:DebugEntry("ScanTradeSkillFrame")

	if (IsTradeSkillLinked()) then return end

	SetTradeSkillSubClassFilter(0, 1, 1)
	SetTradeSkillInvSlotFilter(0, 1, 1)
	ExpandTradeSkillSubClass(1)
	SetTradeSkillItemNameFilter("")
	
	local newRecipes = 0
	local newReagents = 0
	
	for skillId=1, GetNumTradeSkills() do
		local skillName, skillType, numAvailable, isExpanded, altVerb = GetTradeSkillInfo(skillId)
		if (skillType ~= "header" and L.checkForValidSkillName(skillName)) then
			local skillDescription = GetTradeSkillDescription(skillId)
		
			if (not self.db.profile.recipes[skillName]) then
				self.db.profile.recipes[skillName] = { 	scrollItemId = nil,
														scrollName = L.getScrollName(skillName),
														reagents = {},
														description = skillDescription,
														scrollCount = self.db.profile.count,
														numCreated = 0,
														isFiltered = false,
														isUsed = nil,
													 }
				newRecipes = newRecipes + 1
			end
			
			self.db.profile.recipes[skillName].reagents = {}			

			for reagentId=1, GetTradeSkillNumReagents(skillId) do
				local reagentName, reagentTexture, reagentCount, playerReagentCount = GetTradeSkillReagentInfo(skillId, reagentId)

				-- for previously unknown reagents, save the item id
				if (not self.db.profile.reagents[reagentName]) then
					local reagentLink = GetTradeSkillReagentItemLink(skillId, reagentId)
					local _, _, reagentItemId = reagentLink:find("Hitem:(%d+):")
					self.db.profile.reagents[reagentName] = { 	itemId = tonumber(reagentItemId),
																price = 100,
																isFiltered = false,
																isUsed = nil,
															}
					
					newReagents = newReagents + 1
				end
				
				self.db.profile.recipes[skillName].reagents[reagentName] = reagentCount
			end
			
			-- add the vellum as reagent cost (we make scrolls, after all)
			self.db.profile.recipes[skillName].reagents[L.getAppropriateVellum(skillName, skillDescription)] = 1
			
		end
	end
	
	if (newRecipes + newReagents > 0) then
		self:Print("found "..newRecipes.." new recipes and "..newReagents.." new reagents")
	end
	
	self:ApplyFilters()
	self:UpdateRestockStack()

	self:DebugExit("ScanTradeSkillFrame")
end

function Capitalist:BuildList()
	self:DebugEntry("BuildList")

	if (not self.db.profile.reagents[L["Enchanting Vellum"]]) then
		self.db.profile.reagents[L["Enchanting Vellum"]] = { itemId = 38682, price = 0 }
	end

	self.list = {}
	
	for recipe, data in pairs(self.db.profile.recipes) do
		if (data.isUsed == true and data.isFiltered == false) then
			self.list[data.scrollName] = { recipe = recipe }		
		end
	end

	self:DebugExit("BuildList")
end