local L = Capitalist.Locals

function Capitalist:UpdateRestockStack()
	self:DebugEntry("UpdateRestockStack")

	self.inTradeskill = false

	self.restock = {}
	for recipe, data in pairs(self.db.profile.recipes) do
		if (data.isUsed == true and data.isFiltered == false and data.scrollCount > 0) then
			local count = data.scrollCount
			-- search AH and all inventories on that server for this scroll
			for player, t in pairs(self.db.profile.storage[GetCVar("realmName")]) do
				if (t.AH[data.scrollName]) then count = count - t.AH[data.scrollName] end
				if (t.inventory[data.scrollName]) then count = count - t.inventory[data.scrollName] end
			end
			
			if (count > 0) then
				self.restock[recipe] = count
			end
		end
	end
	
	self.requiredReagents = {}
	for recipe, count in pairs(self.restock) do
		for reagent, reagentCount in pairs(self.db.profile.recipes[recipe].reagents) do
			if (self.requiredReagents[reagent] == nil) then
				self.requiredReagents[reagent] = reagentCount * count
			else
				self.requiredReagents[reagent] = self.requiredReagents[reagent] + reagentCount * count
			end
		end
	end

	self:DebugExit("UpdateRestockStack")
end

function Capitalist:ShowRestockStack()
	self:DebugEntry("ShowRestockStack")

	self:UpdateRestockStack()

	local list = {}
	local numRestock = 0
	local reagents = {}
	for k, v in pairs(self.requiredReagents) do reagents[k] = v end
	
	for recipe, count in pairs(self.restock) do
		list[recipe] = count
		numRestock = numRestock + count
	end
	
	-- search for the required reagents in our inventory
	for bag = 0, 4 do
		for slot = 0, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (reagents[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					reagents[name] = reagents[name] - count
				end
			end
		end
	end
	
	-- print out the restock stack
	if (numRestock == 0) then
		self:Print(L["Nothing to restock!"])
		return
	else
		self:Print(string.format(L["Need to restock %d scrolls:"], numRestock))
		for item, count in pairs(list) do
			ChatFrame1:AddMessage("  "..count.."x "..item)
		end
	end
	
	-- print out any reagents missing
	list = {}
	local numMissing = 0
	for reagent, count in pairs(reagents) do
		if (count > 0) then
			list[reagent] = count
			numMissing = numMissing + count
		end
	end
	
	if (numMissing == 0) then
		self:Print(L["Everything available!"])
	else
		self:Print(L["Missing reagents:"])
		for item, count in pairs(list) do
			ChatFrame1:AddMessage("  "..count.."x "..item)
		end
	end

	self:DebugExit("ShowRestockStack")
end

function Capitalist:Restock()
	self:DebugEntry("Restock")

	if (self.inTradeskill) then return end

	SetTradeSkillSubClassFilter(0, 1, 1)
	SetTradeSkillInvSlotFilter(0, 1, 1)
	ExpandTradeSkillSubClass(1)
	SetTradeSkillItemNameFilter("")
	
	for k, v in pairs(self.restock) do
		if (self:TryRestock(k)) then
			return
		end
	end
	
	self:Print(L["Could not restock anything!"])

	self:DebugExit("Restock")
end

function Capitalist:TryRestock(name)
	self:DebugEntry("TryRestock")

	local vellum
	local bag
	local slot

	for reagent in pairs(self.db.profile.recipes[name].reagents) do
		if (reagent:find(L["Enchanting Vellum"])) then
			vellum = reagent
		end
	end
	
	for i = 0, 4 do
		for j = 1, GetContainerNumSlots(i) do
			local link = GetContainerItemLink(i, j)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (name == vellum) then
					bag = i
					slot = j
				end
			end
		end
	end

	if (not bag) then
		return false
	end
	
	for i = 1, GetNumTradeSkills() do
		local skill, _, numAvailable = GetTradeSkillInfo(i)
		if (skill == name and numAvailable > 0) then
			self:DebugMisc(skill)
			DoTradeSkill(i)
			UseContainerItem(bag, slot)
			self.inTradeskill = true
			return self:DebugExit("TryRestock", true)
		end
	end

	return self:DebugExit("TryRestock", false)
end
